@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2016 The ANTLR Project. All rights reserved.
+# Copyright (c) 2012-2017 The ANTLR Project. All rights reserved.
 # Use of this file is governed by the BSD 3-clause license that
 # can be found in the LICENSE.txt file in the project root.
 #/
@@ -13,14 +13,19 @@ from antlr4.atn.ATNDeserializationOptions import ATNDeserializationOptions
 # This is the earliest supported serialized UUID.
 BASE_SERIALIZED_UUID = UUID("AADB8D7E-AEEF-4415-AD2B-8204D6CF042E")
 
+# This UUID indicates the serialized ATN contains two sets of
+# IntervalSets, where the second set's values are encoded as
+# 32-bit integers to support the full Unicode SMP range up to U+10FFFF.
+ADDED_UNICODE_SMP = UUID("59627784-3BE5-417A-B9EB-8131A7286089")
+
 # This list contains all of the currently supported UUIDs, ordered by when
 # the feature first appeared in this branch.
-SUPPORTED_UUIDS = [ BASE_SERIALIZED_UUID ]
+SUPPORTED_UUIDS = [ BASE_SERIALIZED_UUID, ADDED_UNICODE_SMP ]
 
 SERIALIZED_VERSION = 3
 
 # This is the current serialized UUID.
-SERIALIZED_UUID = BASE_SERIALIZED_UUID
+SERIALIZED_UUID = ADDED_UNICODE_SMP
 
 class ATNDeserializer (object):
 
@@ -59,7 +64,13 @@ class ATNDeserializer (object):
         self.readStates(atn)
         self.readRules(atn)
         self.readModes(atn)
-        sets = self.readSets(atn)
+        sets = []
+        # First, read all sets with 16-bit Unicode code points <= U+FFFF.
+        self.readSets(atn, sets, self.readInt)
+        # Next, if the ATN was serialized with the Unicode SMP feature,
+        # deserialize sets with 32-bit arguments <= U+10FFFF.
+        if self.isFeatureSupported(ADDED_UNICODE_SMP, self.uuid):
+            self.readSets(atn, sets, self.readInt32)
         self.readEdges(atn, sets)
         self.readDecisions(atn)
         self.readLexerActions(atn)
@@ -170,8 +181,7 @@ class ATNDeserializer (object):
             s = self.readInt()
             atn.modeToStartState.append(atn.states[s])
 
-    def readSets(self, atn):
-        sets = []
+    def readSets(self, atn, sets, readUnicode):
         m = self.readInt()
         for i in range(0, m):
             iset = IntervalSet()
@@ -181,10 +191,9 @@ class ATNDeserializer (object):
             if containsEof!=0:
                 iset.addOne(-1)
             for j in range(0, n):
-                i1 = self.readInt()
-                i2 = self.readInt()
+                i1 = readUnicode()
+                i2 = readUnicode()
                 iset.addRange(Interval(i1, i2 + 1)) # range upper limit is exclusive
-        return sets
 
     def readEdges(self, atn, sets):
         nedges = self.readInt()
